#ifndef AVIFILE_AUDIODECODER_H
#define AVIFILE_AUDIODECODER_H

/********************************************************
 *
 * Audio decoder interface.
 * This object performs decompression of data
 * from various formats into raw PCM ( 16-bit
 * signed low-endian, suitable for feeding to
 * /dev/dsp ).
 *
 ********************************************************/

#include <formats.h>
#include <infotypes.h>

class IAudioDecoder
{
protected:
    WAVEFORMATEX in_fmt;
    const CodecInfo& record;
    int m_status;
public:
    IAudioDecoder(const CodecInfo& info):record(info){}
    virtual ~IAudioDecoder() {}
    /**
     * Minimal required output buffer size. Calls to Convert() will
     * fail if you pass smaller output buffer to it.
     */
    virtual int GetMinSize() { return 2; }
    /**
     *  Returns output format for this audio decoder.
     */
    virtual int GetOutputFormat(WAVEFORMATEX* destfmt);
    /**
     * Estimates the amount ( in bytes ) of input data that's
     * required to produce specified amount of decompressed PCM data.
     */
    virtual int GetSrcSize(int dest_size){return dest_size;}
    /**
     * Decodes data. It is guaranteed that either size_read or size_written
     * will receive nonzero value if out_size>=GetMinSize().
     */
    virtual int Convert(unsigned char* in_data, unsigned in_size,
			unsigned char* out_data, unsigned out_size,
			unsigned* size_read, unsigned* size_written)	= 0;

    virtual const CodecInfo& GetCodecInfo() const { return record; }
};

#endif // AVIFILE_AUDIODECODER_H
