#ifndef AVIFILE_CPUINFO_H
#define AVIFILE_CPUINFO_H

/**
 *  Utility functions.
 */

#include <default.h>

/**
 *  These two functions return number of ticks ( processor cycles )
 *   since power-on. Return value of first one wraps around each
 *   few seconds ( depends on processor ), but calculations with
 *   32-bit 'int' are a bit faster than with 64-bit 'int64_t'.
 *  They can be used for very precise and fast measurement of time
 *   intervals - call to localcount() takes less than 0.2 mks on 500
 *   MHz processor, and its resolution is measured in nanoseconds.
 *   On systems with properly configured Linux kernel they can be 
 *   used as 3-4 times faster alternative to gettimeofday() 
 *   because they do not require a trip to kernel and back.
 *   On other systems they will probably be even more efficient.
 *  For processors without time-stamp counter feature ( a few
 *   old Pentium-compatible models by Cyrix and maybe others )
 *   these functions are emulated using gettimeofday().
 */
extern unsigned int (*localcount)();
extern int64_t (*longcount)();

/* A thread-save usec sleep */
extern "C" int avi_usleep(unsigned long);

class CPU_Info
{
    double freq;
    int have_tsc;
    int have_mmx;
    static double old_freq();
    static int64_t longcount_notsc();
    static int64_t longcount_tsc();
    static unsigned int localcount_notsc();
    static unsigned int localcount_tsc();
public:
    CPU_Info();
    /**
     * Returns nonzero if the processor supports MMX instruction set.
     */
    int HaveMMX() const {return have_mmx;}
    /**
     * Returns nonzero if the processor has time-stamp counter feature.
     */
    int HaveTSC() const {return have_tsc;}
    /**
     * Returns processor frequency in kHz.
     */
    operator double() const {return freq;}
};

extern CPU_Info freq;

/**
 *  Returns duration of time interval between two timestamps, received
 *  with longcount().
 */
inline float to_float(int64_t t2, int64_t t1)
{
//  int64_t qw=t2-t1;
//    qw=(int64_t)(qw/freq);//milliseconds;
//    return ((int)qw)/1000.;
	return float((t2-t1)/freq/1000.);
}

#endif // AVIFILE_CPUINFO_H
