#ifndef AVIFILE_RENDERER_H
#define AVIFILE_RENDERER_H

/********************************************************

	Video renderer
	Copyright 2000 Eugene Kuznetsov (divx@euro.ru)

*********************************************************/

/*
 *  Goal is to have various Renderers:
 *  for framebuffer, XFree, OpenGL, xv, file (just like mplayer)
 *
 *  so we don't want to have any XFree specific staff here
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <image.h>

class PlayerWidget;

/**
 * Function that returns actual bit depth of specified X display.
 * Should be used instead of any other functions with similar
 * purpose ( DefaultDepth(), x11Depth() in Qt ) when determining
 * bit depth of data to pass to renderer.
 */


class VideoRenderer
{
protected:
    int m_w, m_h;
    int pic_w, pic_h;
    bool subtitles;
public:
    /**
     * Allow YUV renderer to use software YUV->RGB conversion
     * ( slow, only for testing purposes )
     */
    static bool allow_sw_yuv;
    /**
     * Allow renderers to perform resizing window from caller thread
     */
    static bool direct_resizes;

    VideoRenderer(int width, int height, bool _subtitles = false)
	: m_w(width), m_h(height), pic_w(width), pic_h(height), subtitles(_subtitles)
    {
    }

    /**
     * Find the depth
     */
    //virtual int GetDepth() const     				= 0;

    // FIXME: add this method later
    //virtual void EnableSubtitles(bool enable) const		= 0;
    /**
     * Issue the request to draw the image in the window.
     */
    virtual int Draw(const CImage* data)			= 0;
    /**
     * Force all previous X server requests to finish.
     */
    virtual int Sync()						= 0;
    /**
     * Resize renderer window and stretch the image in it.
     */
    virtual int Resize(int& new_w, int& new_h)			= 0;
    /**
     * Toggle on/off fullscreen mode. maximize=true means that
     * renderer should stretch the picture to dimensions of 
     * screen when going fullscreen.
     */
    virtual int ToggleFullscreen(bool maximize = false)		= 0;
    /**
     * Draw a line of subtitles below the picture.
     */
    virtual int DrawSubtitles(const char*)			= 0;
    /**
     * Redraw image (resize, move with pause)
     */
    virtual int Refresh()                                       = 0;
    virtual ~VideoRenderer(){}
};

#ifndef WITHOUT_X
#include <X11/Xlib.h>

int GetPhysicalDepth(Display* dpy);

/**
 * Creates SDL video renderer for RGB data in current depth of
 * display from 2nd argument. Last argument specifies whether
 * this renderer should reserve place for subtitles or not.
 */
VideoRenderer* CreateFullscreenRenderer(PlayerWidget*, Display*,
					int width, int height, bool sub = false);
/**
 * Creates SDL video renderer for YUV data in format yuv_fmt.
 * If it fails ( e.g. because yuv_fmt is not supported by X server
 * and VideoRenderer::allow_sw_yuv==false ), returns pointer to
 * 'ordinary' fullscreen renderer and puts 0 into yuv_fmt.
 */    
VideoRenderer* CreateYUVRenderer(PlayerWidget*, Display*,
				 int width, int height,
				 unsigned long yuv_fmt, bool sub = false);

VideoRenderer* CreateXvYUVRenderer(PlayerWidget*, Display*,
				   int width, int height,
				   unsigned long yuv_fmt, bool sub = false);
#endif // WITHOUT_X

#ifdef HAVE_LIBXXF86DGA
//VideoRenderer* CreateDGARenderer();
#endif

#endif // AVIFILE_RENDERER_H
